/* Copper bar sample, C version  by Anders Montonen						*/
/* This code is in the public domain. 									*/
/* Compiled with the GNU toolkit by KPIT, http://www.kpit.com/			*/
/* You'll also need the crt0.S and linkscript from my site,				*/
/* http://www.helsinki.fi/~ammonton/sega/								*/
/* Remember to assemble crt0.S before compiling this program!			*/
/* Run with SSFEXE from 0x060040000 as usual.							*/

/* Hardware constants */
#include "include/vdp2.h"

/* Sine table for bounce */
const unsigned short SinTable[] = { 115 , 121 , 126 , 131 , 137 , 142 , 147 , 152,
									157 , 162 , 166 , 171 , 175 , 179 , 184 , 187,
									191 , 195 , 198 , 201 , 204 , 207 , 209 , 211,
									213 , 215 , 216 , 217 , 218 , 219 , 219 , 219,
									219 , 219 , 218 , 217 , 216 , 215 , 213 , 211,
									209 , 207 , 204 , 201 , 198 , 195 , 191 , 187,
									184 , 179 , 175 , 171 , 166 , 162 , 157 , 152,
									147 , 142 , 137 , 131 , 126 , 121 , 115 , 110,
									105 , 99 , 94 , 89 , 83 , 78 , 73 , 68,
									63 , 58 , 54 , 49 , 45 , 41 , 36 , 33,
									29 , 25 , 22 , 19 , 16 , 13 , 11 , 9,
									7 , 5 , 4 , 3 , 2 , 1 , 1 , 1,
									1 , 1 , 2 , 3 , 4 , 5 , 7 , 9,
									11 , 13 , 16 , 19 , 22 , 25 , 29 , 33,
									36 , 41 , 45 , 49 , 54 , 58 , 63 , 68,
									73 , 78 , 83 , 89 , 94 , 99 , 105 , 110 };


/* Copperbar as RGB values */
const unsigned short Copperbar[] = { 0x4000, 0x4000, 0x4000, 0x6100, 0x6100,
							0x4000, 0x6100, 0x7e00, 0x6100, 0x7e00,
							0x7e00, 0x6100, 0x7e00, 0x6100, 0x4000,
							0x6100, 0x6100, 0x4000, 0x4000, 0x4000 };

int main(void)
{
	int i;
	/* Init video mode, turn on all backgrounds and put back screen */
	/* into line mode. Point back screen to start of VDP2 RAM */
	TVMD = 0x8110;
	EXTEN = 0;
	BGON = 0;
	LCTAU = 0;
	LCTAL = 0;
	BKTAL = 0;
	BKTAU = 0x8000;
	
	/* Transfer line table into VDP2 memory */
	
	/* The whole screen is 480 rows high, and the copperbar is 20 rows high, */
	/* so we'll put 220 black rows before and after the row */
	/* The back screen uses 15-bit colour, so we'll write a short per row. */
	for (i = 0; i < 220; i++)
	{
		*((u16*)VDP2_RAM_BASE+i) = 0;
	}
	for (i = 0; i < 20; i++)
	{
		*((u16*)VDP2_RAM_BASE+220+i) = Copperbar[i];
	}
	for (i = 0; i < 220; i++)
	{
		*((u16*)VDP2_RAM_BASE+240+i) = 0;
	}
	i = 0;
	for (;;)
	{
		/* Wait for VBLANK */
		while( (TVSTAT & 8) == 0)
			;
		/* Point back screen to start of VDP2 RAM + sine value */
		BKTAL = SinTable[i];
		i++;
		i &= 0x7f;
		/* Wait until we're out of VBLANK */
		while( (TVSTAT & 8) != 0)
			;
	}
}
